/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: chain-dvd.c,v 1.23 2006/07/02 19:41:52 lordjaxom Exp $
 */

#include "chain-dvd.h"
#include "burn.h"
#include "logger-vdr.h"
#include "jobs.h"
#include "render.h"
#include "config.h"
#include "menuburn.h"
#include "filter.h"
#include "setup.h"
#include "etsi-const.h"
#include "boost/bind.hpp"
#include "proctools/shellprocess.h"
#include "proctools/functions.h"
#include "proctools/logger.h"
#include "proctools/format.h"
#include "proctools/extract.h"
#include <string>
#include <sstream>
#include <algorithm>
#include <memory>
#include <vdr/recording.h>
#include <sys/stat.h>

namespace vdr_burn
{
	using namespace std;
	using boost::bind;
	using proctools::shellprocess;
	using proctools::logger;
	using proctools::process;
	using proctools::format;

	chain_dvd::chain_dvd(job& job_):
			chain_vdr( "DVD", job_ ),
			m_step( build ),
			m_currentRecording( get_recordings().begin() ),
			m_pxAudioIndex( 0 ),
			m_totalDone( 0 ),
			m_lastRecSize( 0 ),
			m_lastProgress( 0 )
	{
	}

	void chain_dvd::process_line(const string& line)
	{
		static const string px_found_id( "-> found PES-ID " );
		static const string stat_vobu( "STAT: VOBU " );
		static const string stat_fixing( "STAT: fixing VOBU at " );
		static const string burn_done( "% done, estimate finish" );

		switch (m_step) {
		case build:
			{
				if (line.substr(0, px_found_id.length()) == px_found_id) {
					istringstream parser( line.substr(px_found_id.length()) );
					int cid;
					parser >> setbase(16) >> cid;

					logger::info(format("px found id: {0}") % format::base(cid, 16));
					if (cid >= 0xc0 && cid <= 0xdf) { // audio stream
						ostringstream filename;
						filename << "vdrsync";
						if (m_pxAudioIndex > 0)
							filename << m_pxAudioIndex;
						filename << ".mpa";
						m_currentRecording->set_track_path(cid, filename.str());
						++m_pxAudioIndex;
					}
				}
				else if (line.substr(0, stat_vobu.length()) == stat_vobu) {
					if (!elapsed_since(m_lastProgress, 1))
						return;

					istringstream parser(line.substr(stat_vobu.length()));
					int mbDone;
					parser >> mbDone;
					parser.ignore(4); // " at "
					parser >> mbDone;

					double done = (100.f * m_totalDone
								  + 90.f * get_job().get_requant_factor() * mbDone * MEGABYTE(1))
								  / get_job().get_tracks_size() / 2.f;

					set_progress(static_cast<int>(done));
				}
				else if (line.substr(0, stat_fixing.length()) == stat_fixing) {
					if (!elapsed_since(m_lastProgress, 1))
						return;

					istringstream parser(line.substr(stat_fixing.length()));
					int mbDone;
					parser >> mbDone;

					double done = (100.f * (m_totalDone - m_lastRecSize)
								  + 90.f * m_lastRecSize
								  + 10.f * get_job().get_requant_factor() * mbDone * MEGABYTE(1))
								  / get_job().get_tracks_size() / 2.f;

					set_progress(static_cast<int>(done));
				}
			}
			break;

		case burn:
			{
				istringstream parser(line);
				double percentDone;
				parser >> percentDone;
				char rest[burn_done.length() + 1];
				parser.get(rest, burn_done.length() + 1, '\n');
				if (rest == burn_done) {
					set_progress(static_cast<int>(50.f + percentDone / 2.f));
					set_burn_progress(static_cast<int>(percentDone));
				}
			}
			break;
		};
	}

	bool chain_dvd::initialize(void)
	{
		return prepare_job();
	}

	bool chain_dvd::finished(const process* proc)
	{
		if (proc->return_status() == process::ok) {
			// positive return
			if (proc->name() == "demux" && setup::get().DemuxType == demuxtype_projectx)
				prepare_mplex();
			else if (proc->name() == "mplex") {
				m_lastRecSize = m_currentRecording->get_tracks_size();
				m_totalDone += m_lastRecSize;
				if (m_step == build) {
					// prevent this to happen again when author quits before mplex
					++m_currentRecording;
					if (m_currentRecording != get_recordings().end())
						prepare_recording();
				}
			}
			else if (proc->name() == "author") {
				m_step = burn;
				if(get_job().get_dmh_archive_mode()) {
				  m_currentRecording = get_recordings().begin();
				  return prepare_dmh_archive();
				} else {
				  // no DMH-archive wanted, so go directly to burning
				  return prepare_burning();
				}
			}
			else if (proc->name() == "dmharchive") {
				++m_currentRecording;
				if (m_currentRecording != get_recordings().end()) {
					return prepare_dmh_archive();
				} else {
					return prepare_archive_mark();
				}
			}
			else if (proc->name() == "archivemark") {
				return prepare_burning();
			}
			return true;
		}

		// process failed
		if (proc->name() == "burn" && prepare_device())
			return prepare_burning();

		return false;
	}

	void chain_dvd::create_files(void)
	{
		for_each(get_recordings().begin(), get_recordings().end(),
				 bind( &chain_dvd::create_files, this, _1 ));
	}

	void chain_dvd::create_files(const recording& rec)
	{
		// temp folders
		make_dir(rec.get_paths().temp);
		make_dir(rec.get_paths().data);

		// fifos for video track and muxed movie
		make_fifo(rec.get_video_track_path());
		make_fifo(rec.get_movie_path());
		make_fifo(rec.get_requant_path());

		// fifos for each audio track
		const_track_filter audioTracks( rec.get_tracks(), track_info::streamtype_audio, track_predicate::used );
		for_each(audioTracks.begin(), audioTracks.end(),
				 bind( &chain_dvd::make_fifo, this,
				 	   bind( &recording::get_track_path, rec, _1 )
				));

	}

	bool chain_dvd::prepare_job(void)
	{
		create_files();

		// prepare first recording
		prepare_recording();

		if (get_job().get_disk_type() == disktype_dvd_menu) {
			// render menu's
			cBurnRender render(get_job());
			execute(render);
			if (render.return_status() != process::ok) {
				logger::error("rendering menu failed - aborting job");
#warning error handling
				return false;
			}
		}

		dvdauthor_xml xml(get_job());
		xml.write();

		// start authoring
		shellprocess* author = new shellprocess("author",
												"vdrburn-dvd.sh author");
		make_dir(xml.get_author_path());
		author->put_environment("DVDAUTHOR_XML",  xml.get_xml_path());
		author->put_environment("DVDAUTHOR_PATH", xml.get_author_path());
		author->put_environment("ISO_FILE",       get_job().get_iso_path());
		author->put_environment("DVD_DEVICE",     BurnParameters.DvdDevice);
		add_process(author);
		return true;
	}

	void chain_dvd::prepare_recording(void)
	{
		prepare_demux();
		if (setup::get().DemuxType != demuxtype_projectx)
			prepare_mplex();
	}

	void chain_dvd::prepare_demux(void)
	{
		string demux_call( "vdrburn-dvd.sh " );
		switch (setup::get().DemuxType) {
		case demuxtype_vdrsync: demux_call += "demux"; break;
		case demuxtype_projectx: m_pxAudioIndex = 0; demux_call += "demuxpx"; break;
		}

		// processes
		shellprocess* demux = new shellprocess("demux", demux_call);
		demux->put_environment("RECORDING_PATH", m_currentRecording->get_filename());
		demux->put_environment("IGNORE_TRACKS",  m_currentRecording->get_ignored_cids());
		demux->put_environment("USED_TRACKS",    m_currentRecording->get_used_cids());
		demux->put_environment("MPEG_TMP_PATH",  m_currentRecording->get_paths().temp);
		demux->put_environment("MPEG_DATA_PATH", m_currentRecording->get_paths().data);
		demux->put_environment("CONFIG_PATH",    plugin::get_config_path());
		if (get_job().get_cut_on_demux() && prepare_cutmarks())
			demux->put_environment("USE_CUTTING", "1");
		add_process(demux);
	}

	void chain_dvd::prepare_mplex(void)
	{
		if (get_job().get_requant_factor() > 1) {
			string requant_call( "vdrburn-dvd.sh " );
			switch (setup::get().RequantType) {
			case requanttype_metakine: requant_call += "requant"; break;
			case requanttype_transcode: requant_call += "tcrequant"; break;
			}

			shellprocess* requant = new shellprocess("requant", requant_call);
			requant->put_environment("VIDEO_FILE",     m_currentRecording->get_video_track_path());
			requant->put_environment("REQUANT_FILE",   m_currentRecording->get_requant_path());
			requant->put_environment("REQUANT_FACTOR", get_job().get_requant_factor());
			requant->put_environment("VIDEO_SIZE",     m_currentRecording->get_tracks_size( track_info::streamtype_video ));
			add_process(requant);
		}

		shellprocess* mplex = new shellprocess("mplex", "vdrburn-dvd.sh mplex");
		mplex->put_environment("MOVIE_FILE",     m_currentRecording->get_movie_path());
		mplex->put_environment("MPEG_DATA_PATH", m_currentRecording->get_paths().data);
		mplex->put_environment("MPEG_TMP_PATH",  m_currentRecording->get_paths().temp);
		mplex->put_environment("VIDEO_FILE",
				get_job().get_requant_factor() > 1
					? m_currentRecording->get_requant_path()
					: m_currentRecording->get_video_track_path());
		mplex->put_environment("AUDIO_FILES",    m_currentRecording->get_audio_paths());
		add_process(mplex);
	}

	bool chain_dvd::prepare_device(void)
	{
		shellprocess eject( "eject", "eject " + BurnParameters.DvdDevice );
		execute(eject);
		if (eject.return_status() != process::ok)
			return false;

		Skins.QueueMessage(mtInfo, tr("Please enter the burn plugin"), 10, -1);
		bool result = menu::pagebase::wait_for_user(tr("Please insert an empty disk and press Ok"));
		Skins.QueueMessage(mtInfo, 0, 0, 0);
		return result;
		/*if (get_job().get_store_mode() == smCreateISO)
			return true;

		while (true) {
			wrappers::cdio::device cdio(BurnParameters.DvdDevice);
			if (!cdio && errno != ENOMEDIUM) {
				logger::error(format("error while accessing {0}")
							  % BurnParameters.DvdDevice);
				return false;
			}

			switch (cdio.get_media_status()) {
			case wrappers::cdio::writeable:
			case wrappers::cdio::unknown:
				return true;

			case wrappers::cdio::no_dvd:
			case wrappers::cdio::readable:
				if (!cdio.eject()) {
					logger::error(format("error while ejecting {0}")
								  % BurnParameters.DvdDevice);
					return false;
				}

			case wrappers::cdio::error:
				Skins.QueueMessage(mtInfo, tr("Please enter the burn plugin"),
								   10, -1);
				if (!menu::pagebase::wait_for_user(tr("Please insert an empty "
						 						      "disk and press Ok")))
					return false;
				break;

			case wrappers::cdio::unknown:
				logger::error(format("unknown condition while accessing {0}")
							  % BurnParameters.DvdDevice);
				return false; XXX Double Layer
			}
		}
		return true;*/
	}

	bool chain_dvd::prepare_burning(void)
	{
		switch (get_job().get_store_mode()) {
		case storemode_create:
			{
				shellprocess* burn = new shellprocess("burn",
													  "vdrburn-dvd.sh mkiso");
				burn->put_environment("DVDAUTHOR_PATH",
								  dvdauthor_xml::get_author_path(get_job()));
				burn->put_environment("ISO_FILE", get_job().get_iso_path());
				burn->put_environment("DISC_ID", get_job().get_volume_id());
				add_process(burn);
			}
			break;

		case storemode_burn:
			{
				const char* cmdline;
				if (get_job().get_disk_size() == disksize_cdr)
					cmdline = "vdrburn-dvd.sh burndircd";
				else
					cmdline = "vdrburn-dvd.sh burndir";

				shellprocess* burn = new shellprocess("burn", cmdline);
				burn->put_environment("DVDAUTHOR_PATH",
								  dvdauthor_xml::get_author_path(get_job()));
				burn->put_environment("DVD_DEVICE", BurnParameters.DvdDevice);
				burn->put_environment("BURN_SPEED", setup::get().BurnSpeed);
				burn->put_environment("DISC_ID", get_job().get_volume_id());
				add_process(burn);
			}
			break;

		case storemode_createburn:
			{
				string fifo = format("{0}/burnfifo") % get_paths().temp;
				make_fifo(fifo);

				shellprocess* pipe = new shellprocess("pipe",
													  "vdrburn-dvd.sh pipeiso");
				pipe->put_environment("DVDAUTHOR_PATH",
								  dvdauthor_xml::get_author_path(get_job()));
				pipe->put_environment("ISO_FILE",   get_job().get_iso_path());
				pipe->put_environment("ISO_PIPE",   fifo);
				pipe->put_environment("DISC_ID", get_job().get_volume_id());
				add_process(pipe);

 				char *cmdline;
 				if (get_job().get_disk_size() == disksize_cdr)
 					cmdline = "vdrburn-dvd.sh burnisocd";
 				else
 					cmdline = "vdrburn-dvd.sh burniso";

 				shellprocess* burn = new shellprocess("burn", cmdline);
				burn->put_environment("DVD_DEVICE", BurnParameters.DvdDevice);
				burn->put_environment("ISO_PIPE",   fifo);
				burn->put_environment("BURN_SPEED", setup::get().BurnSpeed);
				add_process(burn);
			}
			break;
		}
		set_is_burning();
		return true;
	}

	bool chain_dvd::prepare_cutmarks(void)
	{
		if (setup::get().DemuxType == demuxtype_vdrsync)
			return true;

		cMarks marks;
		if (!marks.Load(m_currentRecording->get_filename().c_str()) || !marks.Count())
			return false;

		// convert marks to bytepos
		cIndexFile ifile( m_currentRecording->get_filename().c_str(), false );
		string cutfilename( format("{0}/px.cut") % m_currentRecording->get_paths().data );
		ofstream cutfile( cutfilename.c_str() );

		for (cMark *mark = marks.First(); mark != 0; mark = marks.Next(mark)) {
			int pos = mark->position;

			uchar filenumber;
			int offset;
			ifile.Get(pos, &filenumber, &offset, 0, 0);

			uint64_t bytepos = offset;
			for (int i = 1; i < filenumber; ++i) {
				string vdrfile( format("{0}/{1}.vdr") % m_currentRecording->get_filename().c_str()
						% format::prec(format::fill(i, '0'), 3) );
				struct stat filebuf;
				if ((stat(vdrfile.c_str(), &filebuf)) < 0) {
					logger::error(format("error determining size of \"{0}\"") % vdrfile);
					return false; // so px won't complain about missing marks
				}
				bytepos += filebuf.st_size;
			}
			logger::info(format("mark at {0}, bytepos {1}") % *IndexToHMSF(pos, true) % bytepos);
			cutfile << bytepos << endl;
		}
		cutfile.close();
		return true;
	}

	bool chain_dvd::prepare_dmh_archive(void)
	{
		shellprocess* dmharchive = new shellprocess("dmharchive", "vdrburn-dvd.sh dmharchive");
		dmharchive->put_environment("DVDAUTHOR_PATH",
							dvdauthor_xml::get_author_path(get_job()));
		dmharchive->put_environment("RECORDING_PATH", m_currentRecording->get_filename());
		dmharchive->put_environment("TEMP_PATH",  m_currentRecording->get_paths().temp);
		dmharchive->put_environment("CONFIG_PATH", plugin::get_config_path());

		string track = format("{0}") % format::prec(format::fill(std::distance(get_recordings().begin(), m_currentRecording)+1, '0'), 2);
		dmharchive->put_environment("TRACK_ON_DVD", track);

		add_process(dmharchive);
		return true;
	}

	bool chain_dvd::prepare_archive_mark(void)
	{
		shellprocess* archivemark = new shellprocess("archivemark", "vdrburn-dvd.sh archivemark");
		archivemark->put_environment("DVDAUTHOR_PATH", dvdauthor_xml::get_author_path(get_job()));
		archivemark->put_environment("CONFIG_PATH",    plugin::get_config_path());

		add_process(archivemark);
		return true;
	}
}
