/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: common.c,v 1.21 2006/07/01 18:30:43 lordjaxom Exp $
 */

#include "common.h"
#include "setup.h"
#include "proctools/logger.h"
#include "proctools/format.h"
#include "boost/bind.hpp"
#include <sstream>
#include <string>
#include <iterator>
#include <algorithm>
#include <iomanip>
#include <ctime>
#include <vdr/recording.h>
#include <vdr/config.h>
#include <sys/types.h>
#include <glob.h>

namespace vdr_burn
{

	using namespace std;
	using proctools::logger;
	using proctools::format;
	using boost::bind;

	// --- disktype -----------------------------------------------------------

	const char* disktype_strings[disktype_count] =
			{ "DVD with menus", "DVD without menus", "Archive disk" };

	// --- storemode ----------------------------------------------------------

	const char* storemode_strings[storemode_count] =
			{ "Create ISO only", "Write to Disc", "Create ISO and write" };

	// --- chaptersmode -------------------------------------------------------

	const char* chaptersmode_strings[chaptersmode_count] =
			{ "No chapters", "Use editing marks", "Every 5 minutes", "Every 10 minutes",
			  "Every 15 minutes", "Every 30 minutes", "Every hour" };

	const int chaptersmode_intervals[chaptersmode_count] =
			{ -1, -1, 5, 10, 15, 30, 60 };

	// --- disksize -----------------------------------------------------------

	const char* disksize_strings[disksize_count] =
			{ "Single Layer", "Double Layer", "CD-R", "Custom" };

	const int disksize_values[disksize_count] =
			{ 4472, 7944, 690, -1 };

	// --- demuxtype ----------------------------------------------------------

	const char* demuxtype_strings[demuxtype_count] =
			{ "VDRSync", "ProjectX" };

	// --- requanttype --------------------------------------------------------

	const char* requanttype_strings[requanttype_count] =
			{ "M2VRequantizer", "Transcode" };

	// --- string_list --------------------------------------------------------

	string_list::string_list(const string_list& rhs):
			m_strings( rhs.m_strings )
	{
		reset();
	}

	string_list::~string_list()
	{
		delete[] m_cstrings;
	}

	string_list& string_list::operator=(const string_list& rhs)
	{
		delete[] m_cstrings;
		m_strings = rhs.m_strings;
		reset();
		return *this;
	}

	int string_list::find_index(const std::string& value) const
	{
		list_type::const_iterator it = find( m_strings.begin(), m_strings.end(), value );
		if (it != m_strings.end())
			return distance( m_strings.begin(), it );
		return -1;
	}

	void string_list::reset(void)
	{
		m_cstrings = new const char*[m_strings.size()];
		transform( m_strings.begin(), m_strings.end(), m_cstrings, bind( &string::c_str, _1 ) );
	}

const char *TitleChars = "abcdefghijklmnopqrstuvwxyz"
						 "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
						 "0123456789"
						 ":- ";

int ScanPageCount(const std::string& Path)
{
	cString path = AddDirectory(Path.c_str(), "menu-bg-*.png");
	glob_t globbuf;
	int res;

	if ((res = glob(path, 0, NULL, &globbuf)) != 0) {
		const char *error = "Unknown error";

		switch (res) {
			case GLOB_NOSPACE: error = "Out of memory"; break;
			case GLOB_ABORTED: error = "I/O error"; break;
			case GLOB_NOMATCH: error = "File not found"; break;
		}

		esyslog("ERROR[burn]: Error while scanning %s: %s", *path, error);
		return -1;
	}

	res = globbuf.gl_pathc;
	globfree(&globbuf);
	return res;
}

std::string progress_bar(double current, double total, int length)
{
	if (length < 2)
		return "";

	if (current > total)
		current = total;

	int l = static_cast<int>(current * (length - 2) / total);
	ostringstream builder;

	builder << '[';
	fill_n(ostream_iterator<char>(builder), l, '|');
	fill_n(ostream_iterator<char>(builder), length - 2 - l, ' ');
	builder << ']';
	return builder.str();
}

string escape_iso_to_utf8(const string& Text)
{
	stringstream result;
	for (const uchar *in = reinterpret_cast<const uchar*>(Text.c_str());
		 *in; ++in) {
		if (setup::get().UseIso2UTF && *in >= 0x80)
			result << static_cast<char>( ((*in >> 6) & 0x1F) | 0xC0 )
				   << static_cast<char>( (*in & 0x3F) | 0x80 );
		else if (*in == '-') {
			if (setup::get().UseIso2UTF)
				result << static_cast<char>( 0xe2 )
					   << static_cast<char>( 0x80 )
					   << static_cast<char>( 0x93 );
			else
				result << '_';//XXX
		}
		else if (*in == 0x09)
			result << ' ';
		else
			result << static_cast<char>(*in);
	}
	return result.str();
}

void trim_left(char *Text, const char *Chars)
{
	char *tmp = Text, *ptr;
	do {
		ptr = tmp;
		while (strchr(Chars, *ptr) != NULL)
			++ptr;
		memmove(tmp, ptr, strlen(ptr) + 1);
		if ((tmp = strchr(tmp, '~')) != NULL)
			++tmp;
	} while (tmp != NULL);
}

string get_recording_datetime(const cRecording* recording_, char delimiter)
{
	string title = recording_->Title('\t', false, -1);

	string::iterator it = title.begin();
	for (int i = 0; i < 2 && it != title.end(); ++i)
		it = find(it + 1, title.end(), '\t');

	title.erase(it, title.end());
	if (title[title.length() - 1] == ' ')
		title.erase(title.length() - 1);
	if (delimiter != '\t')
		replace(title.begin(), title.end(), '\t', delimiter);
	return title;
}

string get_recording_title(const cRecording* recording_, int level)
{
	stringstream result;
	string name( recording_->Name() );

	if (level == recording_->HierarchyLevels()) {
		string::size_type pos;
		for ( int l = 0; l < level && ((pos = name.find('~')) != string::npos); ++l )
			name.erase(0, pos + 1);

		result << get_recording_datetime(recording_);

		ostringstream archivePath;
		archivePath << recording_->FileName() << "/dvd.vdr";
		if (access(archivePath.str().c_str(), F_OK) == 0)
			result << 'A';
		else
			result << ' ';

		result << (recording_->IsNew() ? '*' : ' ') << '\t' << name;
		return result.str();
	}

	string::size_type lastOffset = 0;
	string::size_type offset;
	while ((offset = name.find('~', lastOffset)) != string::npos) {
		if (level-- == 0)
			break;
		lastOffset = offset + 1;
	}
	result << "\t\t" << name.substr(lastOffset, offset - lastOffset);
	return result.str();
}

std::string get_recording_description(const cRecording* recording_)
{
#if VDRVERSNUM < 10300
	if (recording_->Summary() != 0)
		return recording_->Summary();
#else
	if (recording_->Info()->Description() != 0)
		return recording_->Info()->Description();
	else if (recording_->Info()->ShortText() != 0)
		return recording_->Info()->ShortText();
#endif

	return "";
}

string escape_shellcmd(const string& command)
{
	string result(command);
	replace(result.begin(), result.end(), '/', '_');
	return result;
}

string shell_escape(const string& text, const string& escape)
{
	string result( text );
	for (string::size_type pos = 0; pos < result.length(); ++pos) {
		string::size_type charPos = escape.find(result[pos]);
		if (charPos != string::npos) {
			char replacement[3] = "\\ ";
			replacement[1] = escape[charPos];
			result.replace(pos++, 1, replacement);
		}
	}
	return result;
}

bool elapsed_since(time_t& timestamp, time_t difference)
{
	time_t now = time(NULL);
	if (timestamp + difference <= now) {
		timestamp = now;
		return true;
	}
	return false;
}

string int_to_string(int value, int base, bool prefix)
{
	stringstream conv;
	if (prefix) {
		switch (base) {
		case 2: conv << "0b"; break;
		case 8: conv << "0"; break;
		case 16: conv << "0x"; break;
		}
	}
	conv << setbase(base) << value;

	string result;
	conv >> result;
	return result;
}

}
