/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: common.h,v 1.20 2006/07/01 18:30:43 lordjaxom Exp $
 */

#ifndef VDR_BURN_COMMON_H
#define VDR_BURN_COMMON_H

#include "etsi-const.h"
#include "forward.h"
#include "proctools/functions.h"
#include <string>
#include <vector>
#include <stdint.h>

#define JOBNAMELEN 255
#define MAXATRACKS (MAXAPIDS + MAXDPIDS)
#define MAXPAGETITLES 8

#ifdef DEBUG
#	include <exception>
#	include <stdexcept>
#endif

namespace vdr_burn
{

#ifdef DEBUG
#	define BURN_THROW_DEBUG(msg) throw std::runtime_error( msg );
#else
#	define BURN_THROW_DEBUG(msg)
#endif

#if 0 // maybe later
	// helper class to encapsulate all enumerations into integers, in order to
	// allow editing in VDR's setup items (since enum is not guaranteed to be
	// of type integer)
	template<typename Enum>
	class int_enum
	{
	public:
		int_enum(void) { }
		int_enum(const Enum& value): m_value( value ) { }

		operator Enum() { return Enum( m_value ); }
		operator int&() { return m_value; }
		int_enum<Enum>& operator=(const Enum& value) { m_value = value; return *this; }

	private:
		int m_value;
	};
#endif

	// --- path_pair ----------------------------------------------------------

	struct path_pair
	{
		std::string temp;
		std::string data;
		path_pair(void) {}
		path_pair(const std::string& temp_, const std::string& data_):
			temp(temp_), data(data_) {}
	};

	// --- size_pair ----------------------------------------------------------

	template<typename Ty>
	struct size_pair_t
	{
		typedef Ty size_type;

		size_pair_t(size_type uncut, size_type cut): uncut( uncut ), cut( cut ) { }

		template<typename From>
		size_pair_t(const size_pair_t<From>& from): uncut( from.uncut ), cut( from.cut ) { }

		size_type uncut;
		size_type cut;
	};

	typedef size_pair_t<uint64_t> size_pair;
	typedef size_pair_t<int> length_pair;

	// --- string_list --------------------------------------------------------

	class string_list
	{
	public:
		typedef std::vector<std::string> list_type;

		string_list(void): m_cstrings( 0 ) { }
		template<typename FwdIt>
		string_list(FwdIt first, FwdIt last);
		string_list(const string_list& rhs);
		~string_list();

		string_list& operator=(const string_list& rhs);

		size_t size(void) const { return m_strings.size(); }
		int find_index(const std::string& value) const;

		const char* const* get_strings(void) const { return m_cstrings; }

	private:
		void reset(void);

		list_type m_strings;
		const char** m_cstrings;
	};

	template<typename FwdIt>
	string_list::string_list(FwdIt first, FwdIt last):
			m_strings( first, last )
	{
		reset();
	}

	// --- base_from_member ---------------------------------------------------

	template<typename Type, unsigned int Tag>
	struct base_from_member
	{
		base_from_member(const Type& value): value( value ) { }
		Type* get_pointer(void) { return &value; }
		Type value;
	};

	// --- disktype -----------------------------------------------------------

	enum disktype
	{
		disktype_dvd_menu,
		disktype_dvd_nomenu,
		disktype_archive
	};

	const int disktype_count( disktype_archive + 1 );
	const int disktype_countrequant( disktype_dvd_nomenu + 1 );
	extern const char* disktype_strings[disktype_count];

	// --- storemode ----------------------------------------------------------

	enum storemode
	{
		storemode_create,
		storemode_burn,
		storemode_createburn
	};

	const int storemode_count( storemode_createburn + 1 );
	extern const char* storemode_strings[storemode_count];

	// --- chaptersmode -------------------------------------------------------

	enum chaptersmode
	{
		chaptersmode_none,
		chaptersmode_marks,
		chaptersmode_5,
		chaptersmode_10,
		chaptersmode_15,
		chaptersmode_30,
		chaptersmode_60
	};

	const int chaptersmode_count( chaptersmode_60 + 1 );
	extern const char* chaptersmode_strings[chaptersmode_count];
	extern const int chaptersmode_intervals[chaptersmode_count];

	// --- disksize -----------------------------------------------------------

	enum disksize
	{
		disksize_singlelayer,
		disksize_doublelayer,
		disksize_cdr,
		disksize_custom
	};

	const int disksize_count( disksize_custom + 1 );
	extern const char* disksize_strings[disksize_count];
	extern const int disksize_values[disksize_count];

	// --- demuxtype ----------------------------------------------------------

	enum demuxtype
	{
		demuxtype_vdrsync,
		demuxtype_projectx
	};

	const int demuxtype_count( demuxtype_projectx + 1 );
	extern const char* demuxtype_strings[demuxtype_count];

	// --- requanttype --------------------------------------------------------

	enum requanttype
	{
		requanttype_metakine,
		requanttype_transcode
	};

	const int requanttype_count = requanttype_transcode + 1;
	extern const char* requanttype_strings[requanttype_count];

extern const char *TitleChars;

int ScanPageCount(const std::string& Path);
std::string progress_bar(double current, double total, int length = 20);
std::string escape_iso_to_utf8(const std::string& Text);
void trim_left(char *Text, const char *Chars);
std::string get_recording_datetime(const cRecording* recording_,
								   char delimiter = '\t');
std::string get_recording_title(const cRecording* recording_, int level);
std::string get_recording_description(const cRecording* recording_);
std::string escape_shellcmd(const std::string& command);
std::string shell_escape(const std::string& text, const std::string& escape = "\"$\\");
bool elapsed_since(time_t& timestamp, time_t difference);
std::string int_to_string(int value, int base, bool prefix = false);

template<typename FwdIt, typename Fn>
std::string join_strings(FwdIt first, FwdIt last, Fn func, std::string delimiter)
{
	return proctools::sum(first, last, std::string( "" ), func, delimiter);
}

template<typename FwdIt, typename Fn>
size_pair::size_type accumulate_size(FwdIt first, FwdIt last, Fn func)
{
	return proctools::sum( first, last, size_pair::size_type( 0 ), func );
}

}

#endif // VDR_BURN_COMMON_H
