/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: jobs.h,v 1.24 2006/06/25 18:20:28 lordjaxom Exp $
 */

#ifndef VDR_BURN_JOBS_H
#define VDR_BURN_JOBS_H

#include "common.h"
#include "skins.h"
#include "tracks.h"
#include "forward.h"
#include "proctools/format.h"
#include "proctools/process.h"
#include "proctools/result.h"
#include <deque>
#include <list>
#include <numeric>
#include <string>
#include <memory>

namespace vdr_burn {

	class recording;
	class job;
	class chain_vdr;

	namespace menu
	{
		class track_editor;
		class job_editor;
		class recording_edit_item;
	}

	// --- recording ----------------------------------------------------------

	class recording
	{
	private:
		friend class menu::job_editor;
		friend class menu::track_editor;
		friend class menu::recording_edit_item;

		job*                   m_owner;
		std::string            m_fileName;
		std::string            m_summary;
		std::string            m_datetime;
		path_pair              m_paths;
		char                   m_name[JOBNAMELEN + 1];
		track_info_list        m_tracks;

		size_pair              m_totalSize;
		length_pair            m_totalLength;

    protected:
        friend class recording_scanner;

        void take_tracks(track_info_list& tracks) { m_tracks.swap( tracks ); }
        void set_total_size(const size_pair& value) { m_totalSize = value; }
        void set_total_length(const length_pair& value) { m_totalLength = value; }

	public:
		recording(job* owner, const cRecording *Recording);

		void set_paths(const path_pair& paths, unsigned int index);
		void set_track_path(int cid, const std::string& path);

		const path_pair& get_paths(void) const { return m_paths; }
		const std::string& get_filename(void) const { return m_fileName; }
		const std::string& get_summary(void) const { return m_summary; }
		const std::string& get_datetime(void) const { return m_datetime; }
		const char* get_name(void) const { return m_name; }
		const track_info_list& get_tracks(void) const { return m_tracks; }
		track_info_list& get_tracks(void) { return m_tracks; }

		std::string get_chapters(chaptersmode mode) const;
		std::string get_track_path(const track_info& track) const;
		std::string get_video_track_path(void) const;
		std::string get_requant_path(void) const;
		std::string get_audio_paths(void) const;
		std::string get_used_cids(void) const;
		std::string get_ignored_cids(void) const;
		std::string get_movie_path(void) const;
		std::string get_menu_background(int Page) const;
		std::string get_buttons_normal(void) const;
		std::string get_buttons_highlight(int Page) const;
		std::string get_menu_mpeg(int Page) const;
		std::string get_graft_point(void) const;

		// size information (original recording)
		size_pair::size_type get_total_size(void) const;
		length_pair::size_type get_total_length(void) const;

		// size information (only used tracks)
		size_pair::size_type get_tracks_size(track_info::streamtype type = track_info::streamtype( 0 )) const;
	};

	inline
	std::string recording::get_requant_path(void) const
	{
		return proctools::format("{0}/requant.mpv") % m_paths.temp;
	}

	inline
	std::string recording::get_movie_path(void) const
	{
		return proctools::format("{0}/movie.mpg") % m_paths.temp;
	}

	inline
	std::string recording::get_menu_background(int page) const
	{
		return proctools::format("{0}/menu-bg-{1}.png") % m_paths.data % page;
	}

	inline
	std::string recording::get_buttons_normal(void) const
	{
		return proctools::format("{0}/menu-buttons-ns.png") % m_paths.data;
	}

	inline
	std::string recording::get_buttons_highlight(int page) const
	{
		return proctools::format("{0}/menu-buttons-h-{1}.png")
			   % m_paths.data % page;
	}

	inline
	std::string recording::get_menu_mpeg(int page) const
	{
		return proctools::format("{0}/menu-bg-{1}.mpg") % m_paths.data % page;
	}

	typedef std::list<recording> recording_list;

	// --- job -----------------------------------------------------------

	class job
	{
	private:
		friend class menu::job_editor;

		recording_list m_recordings;
		path_pair m_paths;
		char m_title[JOBNAMELEN + 1];
		int m_diskType;
		int m_dmhArchiveMode;
		int m_storeMode;
		int m_skinIndex;
		int m_chaptersMode;
		int m_diskSize;
		int m_customDiskSizeMB;
		int m_cutOnDemux;
		std::auto_ptr<chain_vdr> m_process;
		double m_factor;

	public:
		job(void);
		virtual ~job();

		void reset(void);  // called by manager when job is re-edited
		void start(void);  // called by manager when job should start
		void stop(void);   // called by manager when job is to be canceled

		proctools::result<bool> append_recording(const cRecording* recording);
		void erase_recording(recording_list::iterator recording);
		void calculate_requant_factor(void);
		recording_list::iterator get_by_filename(const std::string& FileName);
		recording_list& get_recordings(void) { return m_recordings; }
		const recording_list& get_recordings(void) const { return m_recordings; }

		void set_paths(const path_pair& paths);
		void set_title(const char* title) { strncpy(m_title, title, JOBNAMELEN); }

		// progress monitoring
		int get_progress(void) const;
		bool get_is_burning(int& progress) const;
		bool get_is_active(void) const;
		bool get_is_canceled(void) const;

		// information about job facts
		size_pair::size_type get_total_size(void) const;
		size_pair::size_type get_tracks_size(track_info::streamtype type = track_info::streamtype( 0 )) const;

		const path_pair& get_paths(void) const { return m_paths; }
		const char* get_title(void) const { return m_title; }
		disktype get_disk_type(void) const { return disktype( m_diskType ); }
		bool get_dmh_archive_mode(void) const { return m_dmhArchiveMode; }
		storemode get_store_mode(void) const { return storemode( m_storeMode ); }
		chaptersmode get_chapters_mode(void) const { return chaptersmode( m_chaptersMode ); }
		disksize get_disk_size(void) const { return disksize( m_diskSize ); }
		bool get_cut_on_demux(void) const { return m_cutOnDemux; }
		int get_disk_size_mb(void) const;
		proctools::process::status_value get_return_status(void) const;
		const skin& get_skin(void) const { return skin_list::get()[m_skinIndex]; }
		double get_requant_factor(void) const { return m_factor; }
		double get_size_mb(void) const;

		std::string get_iso_path(void) const;
		std::string get_menu_background(int Page) const;
		std::string get_buttons_normal(void) const;
		std::string get_buttons_highlight(int Page) const;
		std::string get_menu_mpeg(int Page) const;
		std::string get_volume_id(void) const;
		std::string get_archive_id(void) const;
	};

	inline
	double job::get_size_mb(void) const
	{
		return static_cast<double>( get_tracks_size() ) / MEGABYTE(1);
	}

	inline
	std::string job::get_menu_background(int page) const
	{
		return proctools::format("{0}/menu-bg-{1}.png") % m_paths.data % page;
	}

	inline
	std::string job::get_buttons_normal(void) const
	{
		return proctools::format("{0}/menu-buttons-ns.png") % m_paths.data;
	}

	inline
	std::string job::get_buttons_highlight(int page) const
	{
		return proctools::format("{0}/menu-buttons-h-{1}.png")
			   % m_paths.data % page;
	}

	inline
	std::string job::get_menu_mpeg(int page) const
	{
		return proctools::format("{0}/menu-bg-{1}.mpg") % m_paths.data % page;
	}

	inline
	std::string job::get_volume_id(void) const
	{
		return std::string( m_title ).substr(0, 32);
	}

	typedef std::deque<job*> job_queue;

}

#endif // VDR_BURN_JOBS_H
