/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: menuburn.c,v 1.35 2006/07/01 18:30:43 lordjaxom Exp $
 */

#include "common.h"
#include "i18n.h"
#include "manager.h"
#include "menuburn.h"
#include "menuitems.h"
#include "setup.h"
#include "skins.h"
#include "filter.h"
#include "etsi-const.h"
#include "proctools/logger.h"
#include "proctools/format.h"
#include "proctools/result.h"
#include "boost/bind.hpp"
#include <algorithm>
#include <iterator>
#include <sstream>
#include <utility>
#include <vdr/interface.h>
#include <vdr/videodir.h>
#include <vdr/menu.h>

namespace vdr_burn
{

	using namespace std;
	using proctools::format;
	using proctools::logger;
	using proctools::process;
	using proctools::result;
	using boost::bind;

	namespace menu
	{

		// --- track_editor ---------------------------------------------------

		track_editor::track_editor(pagebase& parent, recording& recording_):
				m_parent( parent ),
				m_recording( recording_ ),
				m_tracks( recording_.get_tracks() )
		{
			refresh();
		}

		void track_editor::refresh(void)
		{
			int current = Current();
			Clear();
			m_indices.clear();
			for_each(m_tracks.begin(), m_tracks.end(),
					 bind( &track_editor::add_track, this, _1 ));
			SetCurrent(Get(current == -1 ? 0 : current));
			set_help_keys();
			Display();
		}

		bool track_editor::can_move_down(void)
		{
			track_info_list::size_type current( m_indices[Current()] );
			if (current == m_tracks.size() - 1 || m_tracks[current].get_is_video())
				return false;
			return true;
		}

		void track_editor::set_help_keys(void)
		{
			const char* green = 0;
			if (can_move_down())
				green = tr("Move down");
			SetHelp(0, green);
		}

		void track_editor::add_track(track_info& track)
		{
			int index = m_indices.size() == 0 ? 0 : m_indices.back() + 1;

			Add(new bool_item( track.get_type_description(), track.used, tr("Track$Off"), tr("Track$On") ));
			m_indices.push_back(index);

			switch (track.type) {
			case track_info::streamtype_audio:
				Add(new language_item( tr("Language code"), track.language ));
				m_indices.push_back(index);
				break;

			case track_info::streamtype_video:
				Add(new aspect_item( tr("Aspect ratio"), track.video.aspect ));
				m_indices.push_back(index);

			default: break;
			}
		}

		eOSState track_editor::dispatch_key(eKeys key)
		{
			switch (key)
			{
			case kGreen:
				move_down();
				break;

			default:
				break;
			}

			return osContinue;
		}

		void track_editor::move_down(void)
		{
			if (!can_move_down())
				return;

			track_info_list::size_type current( m_indices[Current()] );
			std::swap(m_tracks[current], m_tracks[current + 1]);
			refresh();
		}

		void track_editor::Store(void)
		{
			m_recording.get_tracks() = m_tracks;
		}

		eOSState track_editor::ProcessKey(eKeys key)
		{
			eOSState state = cMenuSetupPage::ProcessKey(key);
			switch (state) {
			case osBack: m_parent.submenu_closing(); return state;
			case osUnknown: return dispatch_key(key);
			default: set_help_keys(); return state;
			}
		}

		// --- title_chooser --------------------------------------------------

		title_chooser::title_chooser(const string& fileName):
				cOsdMenu(tr("Choose title"))
		{
			string::size_type last = 0, pos;
			while ((pos = fileName.find('~', last)) != string::npos) {
				m_data.push_back(fileName.substr(last, pos - last));
				++last;
			}
			refresh();
		}

		void title_chooser::refresh(void)
		{
			Clear();
			for_each(m_data.begin(), m_data.end(), bind( &title_chooser::add_title, this, _1 ));
			Display();
		}

		void title_chooser::add_title(const std::string& title)
		{
			Add(new text_item(title, true));
		}

		eOSState title_chooser::ProcessKey(eKeys key)
		{
			eOSState state = cOsdMenu::ProcessKey(key);
			return state;
		}

		// --- recordings -----------------------------------------------------

		recordings::recordings(void):
				pagebase(tr("Write DVDs - Recordings"), 9, 7),
				m_pathLevel(0),
				m_pathChanged(false),
				m_recordingCount(0)
		{
			display();
		}

		void recordings::display(void)
		{
			cThreadLock RecordingsLock(&Recordings);
			Recordings.Sort();

			int current = Current();
			Clear();
			m_recordingCount = 0;

			if (m_pathChanged) {
				current = -1;
				m_pathChanged = false;
			}

			ostringstream title;
			title << tr("Recordings stored on harddisk");
			if (!m_basePath.empty())
				title << "(" << m_basePath << ")";
			Add(new menu::text_item(title.str()));

			if (manager::get_pending().get_recordings().size() > 0)
				Add(new menu::text_item(
						format(tr("Recordings: {0}, total size: {1} MB"))
						% manager::get_pending().get_recordings().size()
						% format::prec(manager::get_pending().get_size_mb(), 1)));
			else
				Add(new menu::text_item(
						tr("No recordings marked for burning")));

			Add(new menu::text_item(
					progress_bar(manager::get_pending().get_size_mb(),
								 manager::get_pending().get_disk_size_mb(), 100)));
			Add(new menu::text_item(""));

			recording_item *lastItem = 0;
			string lastText;
			for (cRecording *rec = Recordings.First(); rec != NULL; rec = Recordings.Next(rec)) {
				string recName( rec->Name() );

				if (manager::get_pending().get_by_filename(rec->FileName())
						!= manager::get_pending().get_recordings().end())
					continue;

				if (!m_basePath.empty() &&
						(recName.find(m_basePath) != 0 || recName[m_basePath.length()] != '~'))
					continue;

				menu::recording_item* item = new menu::recording_item(rec, m_pathLevel);
				string itemText( item->Text() );
				if (itemText.length() != 0 && (lastItem == 0 || itemText != lastText)) {
					// select directory we are coming from as current item
					if (recName.find(m_lastPath) == 0 && rec->Name()[m_lastPath.length()] == '~') {
						current = count();
						m_lastPath.clear();
					}

					Add(item);
					lastText = itemText;
					lastItem = item;
					++m_recordingCount;
				} else
					delete item;

				if (lastItem != 0 && lastItem->is_directory())
					lastItem->add_entry(rec->IsNew());
			}

			set_current(current);
			pagebase::display();
		}

		void recordings::set_help_keys(void)
		{
			cOsdItem* current = Get(Current());

			string red;
			if (current != 0) {
				if (!dynamic_cast<menu::recording_item*>( current )
						->is_directory())
					red = tr("Choose");
			}
			set_help(red);
		}

		bool recordings::menu_closing(void)
		{
			if (m_pathLevel == 0)
				return true;

			string newPath;
			if (m_pathLevel > 1) {
				string::size_type pos;
				if ((pos = m_basePath.rfind('~')) != string::npos)
					newPath = m_basePath.substr(0, pos);
			}
			m_lastPath = m_basePath;
			m_basePath = newPath;
			m_pathChanged = true;
			--m_pathLevel;
			display();
			return false;
		}

		eOSState recordings::ok_pressed(void)
		{
			recording_item* item =
					dynamic_cast<recording_item*>( Get(Current()) );

			if (item == 0)
				return osUnknown;

			if (!item->is_directory()) {
				string title = item->get_recording()->Title();
				string::size_type pos = title.rfind('~');
				if (pos != string::npos)
					title.erase( pos );
				return AddSubMenu(new cMenuText( title.c_str(),	get_recording_description( item->get_recording() ).c_str() ));
			}

			++m_pathLevel;
			ostringstream newPath;
			if (!m_basePath.empty())
				newPath << m_basePath << "~";
			newPath << item->get_name();
			m_basePath = newPath.str();
			m_pathChanged = true;
			display();
			return osContinue;
		}

		eOSState recordings::red_pressed(void)
		{
			recording_item* item =
					dynamic_cast<recording_item*>( Get(Current()) );

			if (item == 0 || item->is_directory())
				return osUnknown;

			Skins.Message(mtStatus, tr("Analyzing recording..."));
			result<bool> res = manager::get_pending().append_recording(item->get_recording());
			if (!res)
				Skins.Message(mtError, res);
			display();
			if (m_recordingCount == 0 && menu_closing())
				return osBack;
			return osContinue;
		}

		// --- job_editor -----------------------------------------------------

		job_editor::job_editor(void):
				pagebase( tr("Write DVDs - Job"), 16 ),
				m_offset( 0 ),
				m_lastOptions( manager::get_pending() )
		{
			display();
		}

		void job_editor::display(void)
		{
			item_position position( current() );

			clear();

			add_label(tr("Recordings marked for burning"));
			add_label("");
			m_offset = 2;

			if (manager::get_pending().get_recordings().size() > 0) {
				recording_list::iterator rec;
				for (rec = manager::get_pending().m_recordings.begin();
						rec != manager::get_pending().m_recordings.end();
						++rec)
					add_item(rec);

			} else {
				add_label(tr("No recordings marked"), true);
				pagebase::display();
				return;
			}

			add_label("");
			add_label(format(tr("Total size: {0} MB"))
					% format::prec(manager::get_pending().get_size_mb(), 1));
			add_label(progress_bar(manager::get_pending().get_size_mb(),
					manager::get_pending().get_disk_size_mb(), 100));
			//XXX Width of menu

			int diskTypeCount = disktype_count;
			if (manager::get_pending().get_requant_factor() > 1) {
				add_label(tr("Recording(s) would be shrinked!"));
				diskTypeCount = disktype_countrequant;
			}

			add_label("");

			add_item(tr("DVD title"), manager::get_pending().m_title, JOBNAMELEN, TitleChars);
			add_item(tr("Disk type"), manager::get_pending().m_diskType, diskTypeCount,
					i18n::get_disk_types());

			if (setup::get().OfferDmhArchiveMode &&
					manager::get_pending().get_disk_type() != disktype_archive)
				add_item(tr("DMH-archive"), manager::get_pending().m_dmhArchiveMode);

			if ((manager::get_pending().get_disk_type() != disktype_archive &&
					manager::get_pending().m_dmhArchiveMode == 1) ||
					(manager::get_pending().get_disk_type() == disktype_archive))
				add_label(format("{0}:\t{1}")
						% tr("Archive-ID") % manager::get_pending().get_archive_id(), true);

			if (manager::get_pending().get_disk_type() == disktype_dvd_menu &&
					skin_list::get().size() > 1)
				add_item(tr("Skin"), manager::get_pending().m_skinIndex, skin_list::get().size(),
						skin_list::get().get_strings());

			if (setup::get().OfferChapters &&
					manager::get_pending().get_disk_type() != disktype_archive)
				add_item(tr("Chapters"), manager::get_pending().m_chaptersMode, chaptersmode_count,
						i18n::get_chapters_modes());

			if (setup::get().OfferDiskSize)
				add_item(tr("Disk size"), manager::get_pending().m_diskSize, disksize_count,
						i18n::get_disk_sizes());

			if (setup::get().OfferCutOnDemux)
				add_item(tr("Cut"), manager::get_pending().m_cutOnDemux);

			if (!BurnParameters.DvdDevice.empty() && !BurnParameters.IsoPath.empty() &&
					setup::get().OfferStoreMode)
				add_item(tr("Target"), manager::get_pending().m_storeMode, storemode_count,
						i18n::get_store_modes());

			set_current(position);
			pagebase::display();
		}

		void job_editor::set_help_keys(void)
		{
			string red, green, yellow;
			if (manager::get_pending().get_recordings().size() > 0) {
				red = tr("Button$Start");

				item_position position = current();
				if (position.groupId == pagebase::job_recording) {
					yellow = tr("Remove");
					if (position.index - m_offset < manager::get_pending().get_recordings().size() - 1) // XXX
						green = tr("Move down");
				}
			}
			set_help(red, green, yellow);
		}

		eOSState job_editor::menu_update(void)
		{
			job_options options( manager::get_pending() );
			if (m_lastOptions != options) {
				manager::get_pending().calculate_requant_factor();
				display();
			}
			m_lastOptions = options;
			return osContinue;
		}

		eOSState job_editor::ok_pressed(void)
		{
			item_position position( current() );
			if (position.groupId != pagebase::job_recording)
				return osUnknown;

			recording_edit_item* item( static_cast<recording_edit_item*>(Get(position.index)) );
			if (item != 0 && manager::get_pending().get_disk_type() < disktype_countrequant)
				return AddSubMenu(new menu::track_editor( *this, *item->get_recording() ));

			return osUnknown;
		}

		eOSState job_editor::red_pressed(void)
		{
			if (manager::get_pending().get_recordings().size() == 0)
				return osUnknown;

			recording_list::const_iterator rec;
			for (rec = manager::get_pending().get_recordings().begin();
				 rec != manager::get_pending().get_recordings().end(); ++rec) {

				const_track_filter videoTracks( rec->get_tracks(), track_info::streamtype_video, track_predicate::used );
				if (videoTracks.begin() == videoTracks.end()) {
					Skins.Message(mtError, tr("No video track selected!"));
					// XXX Should be moved to job control to determine whether
					// a video track is necessary (audio-dvd or so)
					return osContinue;
				}

				const_track_filter audioTracks( rec->get_tracks(), track_info::streamtype_audio, track_predicate::used );
				if (audioTracks.begin() == audioTracks.end()) {
					Skins.Message(mtError, tr("No audio track selected!"));
					return osContinue;
				}
			}

			if (manager::get_pending().get_size_mb()
					>= FreeDiskSpaceMB(BurnParameters.DataPath.c_str())) {
				Skins.Message(mtError, tr("Not enough free disk space!"));
				return osContinue;
			}

			manager::start_pending();
			starter::set_display(display::status);
			return osBack;
		}

		eOSState job_editor::green_pressed(void)
		{
			item_position position( current() );
			if (position.groupId != pagebase::job_recording)
				return osUnknown;

			recording_edit_item *item = static_cast<recording_edit_item*>( Get(position.index) );
			if (static_cast<recording_list::size_type>(
					distance(manager::get_pending().m_recordings.begin(), item->get_recording())
					) < manager::get_pending().get_recordings().size() - 1) {
				recording_list::iterator thisRec( item->get_recording() );
				recording_list::iterator nextRec( thisRec );
				++nextRec;

				iter_swap(thisRec, nextRec);
				display();
			}
			return osContinue;
		}

		eOSState job_editor::yellow_pressed(void)
		{
			item_position position( current() );
			if (position.groupId != pagebase::job_recording)
				return osUnknown;

			if (!Interface->Confirm(tr("Remove recording from list?")))
				return osContinue;

			recording_edit_item *item = static_cast<recording_edit_item*>( Get(position.index) );

			manager::get_pending().erase_recording(item->get_recording());
			if (manager::get_pending().get_recordings().size() == 0)
				manager::reset_pending();
			display();
			return osContinue;
		}

		void job_editor::submenu_closing(void)
		{
			manager::get_pending().calculate_requant_factor();
		}

		// --- status ---------------------------------------------------------

		status::status(void):
				pagebase(tr("Write DVDs - Status"), 16)
		{
			display();
		}

		void status::display(void)
		{
			int current = Current();
			Clear();

			Add(new menu::text_item(tr("Queued jobs")));
			Add(new menu::text_item());

			manager::lock man_lock;

			int number = 1;
			if (manager::get_active() != 0) {
				Add(new menu::job_item(manager::get_active(), number++));

				int percent;
				if (manager::get_active()->get_is_burning(percent))
					Add(new menu::text_item(
							format(tr("Job active (Writing: {0}%)"))
							% percent));
				else
					Add(new menu::text_item(tr("Job active (Converting)")));

				Add(new menu::text_item(
						progress_bar(manager::get_active()->get_progress(),
									 100, 100)));
				//XXX Width of menu
			} else
				Add(new menu::text_item(tr("No active or waiting jobs present"),
										  true));

			for (job_queue::const_iterator job = manager::get_queued().begin();
				 job != manager::get_queued().end(); ++job)
				Add(new menu::job_item(*job, number++));

			if (manager::get_canceled().size() > 0) {
				Add(new menu::text_item());
				Add(new menu::text_item(tr("Canceled jobs")));
				Add(new menu::text_item());

				number = 1;
				job_queue::const_iterator job;
				for (job = manager::get_canceled().begin();
					 job != manager::get_canceled().end(); ++job)
					Add(new menu::job_item(*job, number++, true));
			}

			if (manager::get_erroneous().size() > 0) {
				Add(new menu::text_item());
				Add(new menu::text_item(tr("Erroneous jobs")));
				Add(new menu::text_item());

				number = 1;
				job_queue::const_iterator job;
				for (job = manager::get_erroneous().begin();
					 job != manager::get_erroneous().end(); ++job) {
					Add(new menu::job_item(*job, number++, true));
				}
			}
			set_current(current);
			pagebase::display();
		}

		void status::set_help_keys(void)
		{
			job_item* item =
					dynamic_cast<job_item*>( Get(Current()) );

			manager::lock man_lock;

			string green, yellow;
			if (item != 0) {
				if (item->get_job()->get_is_canceled())
					green = tr("Restart");
				else if (item->get_job()->get_return_status() == process::error)
					green = tr("Retry");

				if (item->get_job() == manager::get_active())
					yellow = tr("Cancel");
				else
					yellow = tr("Remove");
			}
			set_help(string(), green, yellow);
		}

		eOSState status::menu_update(void)
		{
			display();
			return osContinue;
		}

		eOSState status::green_pressed(void)
		{
			job_item* item =
					dynamic_cast<job_item*>( Get(Current()) );

			if (item == 0)
				return osUnknown;

			manager::lock man_lock;

			if (item->get_job() == manager::get_active())
				return osUnknown;

			if (manager::get_pending().get_recordings().size() > 0 &&
					!Interface->Confirm(tr("Replace current job?")))
				return osContinue;

			manager::replace_pending(item->get_job());
			starter::set_display(display::job);
			return osBack;
		}

		eOSState status::yellow_pressed(void)
		{
			job_item* item =
					dynamic_cast<job_item*>( Get(Current()) );

			if (item == 0)
				return osUnknown;

			manager::lock man_lock;

			if (item->get_job() == manager::get_active()) {
				if (Interface->Confirm(tr("Job still active - really cancel?")))
					item->get_job()->stop();
			}
			else if (Interface->Confirm(tr("Remove job from list?"))) {
				manager::erase_job(item->get_job());
				delete item->get_job();
			}
			display();
			return osContinue;
		}
	}

}
