/*
 * See the files COPYING and README for copyright information and how to reach
 * the author.
 *
 *  $Id: setup.c,v 1.18 2006/07/01 21:10:13 lordjaxom Exp $
 */

#include "setup.h"
#include "manager.h"
#include "menuitems.h"
#include "common.h"
#include "i18n.h"
#include <algorithm>
#include <vdr/menuitems.h>
#include <vdr/videodir.h>
#include <getopt.h>

namespace vdr_burn
{

	using namespace std;
	using namespace proctools;

// --- cBurnParameters --------------------------------------------------------

cBurnParameters BurnParameters;

cBurnParameters::cBurnParameters(void):
		DataPath(VideoDirectory),
		TempPath(TMPDIR),
		DvdDevice(DVDDEV),
		IsoPath(ISODIR)
{
}

bool cBurnParameters::ProcessArgs(int argc, char *argv[])
{
	static struct option opts[] = {
			{ "tempdir", required_argument, NULL, 't' },
			{ "datadir", required_argument, NULL, 'd' },
			{ "dvd",     required_argument, NULL, 'D' },
			{ "iso",     required_argument, NULL, 'i' },
			{ NULL }
	};

	int c, optind = 0;
	while ((c = getopt_long(argc, argv, "t:d:D:i:", opts, &optind)) != -1) {
		switch (c) {
			case 't': TempPath = optarg; break;
			case 'd': DataPath = optarg; break;
			case 'D': DvdDevice = optarg; break;
			case 'i': IsoPath = optarg; break;
			default:  return false;
		}
	}

	struct stat sbuf;
	if (stat(DvdDevice.c_str(), &sbuf) != 0) {
		isyslog("burn: couldn't stat %s, assuming iso-creation only",
				DvdDevice.c_str());
#ifdef STONEAGE_GCC
		DvdDevice = string();
#else
		DvdDevice.clear();
#endif
	}

	if (stat(IsoPath.c_str(), &sbuf) != 0) {
		isyslog("burn: couldn't stat %s, assuming writing to disc only",
				IsoPath.c_str());
#ifdef STONEAGE_GCC
		IsoPath = string();
#else
		IsoPath.clear();
#endif
	}

	if (DvdDevice.empty() && IsoPath.empty()) {
		esyslog("ERROR[burn]: no targets left, check --dvd and --iso");
		fprintf(stderr,
				"ERROR[burn]: no targets left, check --dvd and --iso\n");
		return false;
	}
	return true;
}

// --- setup ------------------------------------------------------------------

setup::setup(void):
		PROCTOOLS_INIT_PROPERTY( UseIso2UTF,          0 ),
		PROCTOOLS_INIT_PROPERTY( RemovePath,          0 ),
		PROCTOOLS_INIT_PROPERTY( OfferChapters,       1 ),
		PROCTOOLS_INIT_PROPERTY( OfferDiskSize,       1 ),
		PROCTOOLS_INIT_PROPERTY( OfferStoreMode,      1 ),
		PROCTOOLS_INIT_PROPERTY( OfferDmhArchiveMode, 1 ),
		PROCTOOLS_INIT_PROPERTY( DiskType,            disktype_dvd_menu ),
		PROCTOOLS_INIT_PROPERTY( SkinIndex,           0 ),
		PROCTOOLS_INIT_PROPERTY( ChaptersMode,        chaptersmode_10 ),
		PROCTOOLS_INIT_PROPERTY( StoreMode,           storemode_burn ),
		PROCTOOLS_INIT_PROPERTY( DmhArchiveMode,      1 ),
		PROCTOOLS_INIT_PROPERTY( DiskSize,            disksize_singlelayer ),
		PROCTOOLS_INIT_PROPERTY( CustomDiskSize,      200 ),
		PROCTOOLS_INIT_PROPERTY( BurnSpeed,           0 ),
		PROCTOOLS_INIT_PROPERTY( DemuxType,           demuxtype_vdrsync ),
		PROCTOOLS_INIT_PROPERTY( CutOnDemux,          0 ),
		PROCTOOLS_INIT_PROPERTY( OfferCutOnDemux,     1 ),
		PROCTOOLS_INIT_PROPERTY( RequantType,         requanttype_metakine ),
		PROCTOOLS_INIT_PROPERTY( PreserveLogFiles,    0 ),
		PROCTOOLS_INIT_PROPERTY( DefaultLanguage,     0 ),
		PROCTOOLS_INIT_PROPERTY( MainMenuStatus,      1 )
{
}

setup& setup::get(void)
{
	static setup instance;
	return instance;
}

// --- cBurnMenuSetupPage helper functors -------------------------------------

struct setup_storer
{
	setup_storer(cBurnMenuSetupPage* menu): m_menu( menu ) {}
	void operator()(pair<const string, property_base*>& value)
	{
		m_menu->SetupStore(value.first.c_str(), value.second->str().c_str());
	}
	cBurnMenuSetupPage* m_menu;
};

// --- cBurnMenuSetupPage -----------------------------------------------------

cBurnMenuSetupPage::cBurnMenuSetupPage(void)
{
	m_data = setup::get();

	Add(new cMenuEditBoolItem(tr("Convert texts to UTF-8"), &m_data.UseIso2UTF));
	Add(new cMenuEditBoolItem(tr("Remove path component"), &m_data.RemovePath));
	Add(new cMenuEditStraItem(tr("Demux using"), &m_data.DemuxType, demuxtype_count,
				i18n::get_demux_types()));
	Add(new cMenuEditStraItem(tr("Requant using"), &m_data.RequantType, requanttype_count,
				i18n::get_requant_types()));
	Add(new cMenuEditBoolItem(tr("Preserve logfiles"), &m_data.PreserveLogFiles));
	Add(new cMenuEditStraItem(tr("Spare language code"), &m_data.DefaultLanguage,
			track_info::get_language_codes().size(), track_info::get_language_codes().get_strings()));
	Add(new cMenuEditBoolItem(tr("Status in main menu"), &m_data.MainMenuStatus));
	Add(new cMenuEditBoolItem(tr("Offer chapters"), &m_data.OfferChapters));
	Add(new cMenuEditBoolItem(tr("Offer target"), &m_data.OfferStoreMode));
	Add(new cMenuEditBoolItem(tr("Offer disk size"), &m_data.OfferDiskSize));
	Add(new cMenuEditBoolItem(tr("Offer cutting"), &m_data.OfferCutOnDemux));
	Add(new cMenuEditBoolItem(tr("Offer DMH-archive"), &m_data.OfferDmhArchiveMode));

	Add(new menu::text_item(tr("--- Defaults --------------------------------"
							   "-------------------------")));
	Add(new cMenuEditStraItem(tr("Disk type"), &m_data.DiskType, disktype_count,
				i18n::get_disk_types()));
	Add(new cMenuEditStraItem(tr("Skin"), &m_data.SkinIndex, skin_list::get().size(),
				skin_list::get().get_strings()));
	Add(new cMenuEditStraItem(tr("Chapters"), &m_data.ChaptersMode, chaptersmode_count,
				i18n::get_chapters_modes()));
	if (BurnParameters.IsoPath.empty() || BurnParameters.DvdDevice.empty()) {
		ostringstream text;
		text << tr("Target") << ":\t";
		if (BurnParameters.IsoPath.empty())
			text << i18n::get_store_modes()[storemode_burn];
		else
			text << i18n::get_store_modes()[storemode_create];
		Add(new menu::text_item(text.str().c_str(), true));
	} else
		Add(new cMenuEditStraItem(tr("Target"), &m_data.StoreMode, storemode_count,
				i18n::get_store_modes()));
	Add(new cMenuEditStraItem(tr("Disk size"), &m_data.DiskSize, disksize_count,
				i18n::get_disk_sizes()));
	Add(new cMenuEditIntItem(tr("Custom disk size (MB)"), &m_data.CustomDiskSize, 0, 9999));
	Add(new menu::speed_edit_item(tr("Burn speed"), m_data.BurnSpeed));
	Add(new cMenuEditBoolItem(tr("Cut"), &m_data.CutOnDemux));
 	Add(new cMenuEditBoolItem(tr("DMH-archive"), &m_data.DmhArchiveMode));
}

void cBurnMenuSetupPage::Store(void)
{
	for_each(m_data.begin(), m_data.end(), setup_storer( this ));
	setup::get() = m_data;
	if (manager::get_pending().get_recordings().size() == 0)
		manager::reset_pending();
}

}
