#include "tracks.h"
#include "adaptor.h"
#include "filter.h"
#include "setup.h"
#include "proctools/format.h"
#include "proctools/logger.h"
#include "boost/bind.hpp"
#include <algorithm>
#include <iterator>
#include <map>
#include <stdexcept> //XXX
#include <vdr/i18n.h>

namespace vdr_burn
{

	using namespace std;
	using proctools::format;
	using proctools::logger;
	using boost::bind;

	// --- track_info ---------------------------------------------------------

	track_info::alpha_pair track_info::language_code_map[] =
			{ alpha_pair("eng", "en"),
			  alpha_pair("deu", "de"),
			  alpha_pair("ger", "de"),
			  alpha_pair("slv", "sl"),
			  alpha_pair("slo", "sk"),
			  alpha_pair("ita", "it"),
			  alpha_pair("dut", "nl"),
			  alpha_pair("nla", "nl"),
			  alpha_pair("nld", "nl"),
			  alpha_pair("por", "pt"),
			  alpha_pair("fra", "fr"),
			  alpha_pair("fre", "fr"),
			  alpha_pair("nor", "no"),
			  alpha_pair("fin", "fi"),
			  alpha_pair("smi", "fi"),
			  alpha_pair("pol", "pl"),
			  alpha_pair("esl", "es"),
			  alpha_pair("spa", "es"),
			  alpha_pair("ell", "el"),
			  alpha_pair("gre", "el"),
			  alpha_pair("sve", "sv"),
			  alpha_pair("swe", "sv"),
			  alpha_pair("rom", "ro"),
			  alpha_pair("rum", "ro"),
			  alpha_pair("hun", "hu"),
			  alpha_pair("cat", "ca"),
			  alpha_pair("cln", "ca"),
			  alpha_pair("rus", "ru"),
			  alpha_pair("hrv", "hr"),
			  alpha_pair("est", "et"),
			  alpha_pair("dan", "da"),
			  alpha_pair("cze", "cs"),
			  alpha_pair("ces", "cs") };

	track_info::track_info(int cid, streamtype type):
			cid( cid ),
			language( get_language_codes().get_strings()[setup::get().DefaultLanguage] ),
			type( type ),
			bitrate( 0 ),
			size( 0, 0 ),
			used( 1 )
	{
	}

	void track_info::set_language(const string& language)
	{
		// convert 3 char code to 2 char code (ISO 639)
		typedef map<string, string> alpha_map;
		static alpha_map alpha( language_code_map, language_code_map + sizeof(language_code_map)/sizeof(language_code_map[0]) );
		static string invalid_language;

		// to prevent that empty elements are created in the map
		alpha_map::iterator it = alpha.find(language);
		if (it == alpha.end()) {
			logger::info(format( "language '{0}' invalid or not found in iso 639, setting default" ) % language);
			this->language = get_language_codes().get_strings()[setup::get().DefaultLanguage];
			return;
		}

		logger::info(format( "language code for '{0}' is '{1}'" ) % language % it->second);
		this->language = it->second;
	}

	const string_list& track_info::get_language_codes(void)
	{
		static string_list alpha;
		if (alpha.size() == 0) {
			vector<string> languages( sizeof(language_code_map)/sizeof(language_code_map[0]) );
			transform( language_code_map, language_code_map + sizeof(language_code_map)/sizeof(language_code_map[0]),
					languages.begin(), bind( &alpha_pair::second, _1 ) );
			languages.erase( unique( languages.begin(), languages.end() ), languages.end() );

			alpha = string_list( languages.begin(), languages.end() );
		}
		return alpha;
	}

	string track_info::get_type_description(void) const
	{
		ostringstream result;

		switch (type) {
		case track_info::streamtype_video:
			result << tr("Video track");
			break;

		case track_info::streamtype_audio:
			result << tr("Audio track");
			break;

		default:
			result << tr("Unknown");
		}
		if (!description.empty())
			result << " (" << description << ")";

		return result.str();
	}

	// --- track_info_list ----------------------------------------------------

	const track_info& track_info_list::operator[](int index) const
	{
		const_iterator it( begin() );
		advance(it, index);
		return *it;
	}

	track_info& track_info_list::operator[](int index)
	{
		iterator it( begin() );
		advance(it, index);
		return *it;
	}

	track_info_list::const_iterator track_info_list::find_cid(int cid) const
	{
        return find_if( begin(), end(), bind( equal_to<int>(), bind( &track_info::cid, _1 ), cid ) );
	}

	track_info_list::iterator track_info_list::find_cid(int cid)
	{
        return find_if( begin(), end(), bind( equal_to<int>(), bind( &track_info::cid, _1 ), cid ) );
	}

}
